<?php

namespace EventPortal\Domain\Event\Form\Models;

use EventPortal\Concerns\Sluggable;
use EventPortal\Domain\Component\Option;
use EventPortal\Domain\Event\Form\Enums\FormType;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Model;
use EventPortal\Domain\Table\Concerns\HasTable;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\Relations\HasOne;
use OwenIt\Auditing\Auditable;
use OwenIt\Auditing\Contracts\Auditable as AuditableInterface;
use Spatie\Enum\Laravel\HasEnums;
use Spatie\Sluggable\HasSlug;
use Spatie\Sluggable\SlugOptions;

class Form extends Model implements AuditableInterface
{
    use HasEnums, Auditable, HasTable, HasSlug, Sluggable;

    public const KEY = 'form';

    /** @var array<string,mixed> */
    protected $enums = [
        'type' => FormType::class.':nullable',
    ];

    /** @var array<string> */
    protected $fillable = [
        'name',
        'slug',
        'type',
        'uuid',
        'event_id',
    ];

    public function event(): BelongsTo
    {
        return $this->belongsTo(Event::class);
    }

    public function steps(): HasMany
    {
        return $this->hasMany(FormStep::class)->orderBy('order');
    }

    public function firstStep(): HasOne
    {
        return $this->hasOne(FormStep::class)->orderBy('order', 'asc');
    }

    public function finalStep(): HasOne
    {
        return $this->hasOne(FormStep::class)->orderBy('order', 'desc');
    }

    public function getSlugOptions(): SlugOptions
    {
        return SlugOptions::create()
            ->generateSlugsFrom('name')
            ->saveSlugsTo('slug');
    }

    public function scopeForEvent(Builder $query, Event $event): Builder
    {
        return $query->where('event_id', $event->id);
    }

    public function scopeRegistration(Builder $query): Builder
    {
        return $query->where('type', FormType::Registration());
    }

    public static function typesAsOptions(): array
    {
        return collect(array_flip(FormType::toArray()))
            ->map(static function ($value): array {
                return Option::make($value)->toArray();
            })
            ->toArray();
    }
}
